<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:template="http://www.boddie.org.uk/ns/xmltools/template"
                version="1.0">

  <xsl:param name="types"/>

  <!-- Add or process type elements inside item elements. -->

  <xsl:template match="item">
    <!-- Copy the element and its contents. -->
    <xsl:copy>
      <xsl:apply-templates select="@*"/>
      <!-- Look for a type element. -->
      <xsl:choose>
        <xsl:when test="type">
          <!-- Use the existing type element. -->
          <xsl:apply-templates select="type"/>
        </xsl:when>
        <xsl:otherwise>
          <!-- Add a type element if none was found. -->
          <xsl:call-template name="type"/>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*[local-name() != 'type']"/>
    </xsl:copy>
  </xsl:template>

  <!-- Investigate type elements. -->

  <xsl:template name="type">
    <!-- Make the element. -->
    <type>
      <xsl:call-template name="type-enum"/>
    </type>
  </xsl:template>

  <xsl:template match="type">
    <!-- Copy the element. -->
    <xsl:copy>
      <xsl:apply-templates select="@*"/>
      <xsl:call-template name="type-enum">
        <!-- Use the value of the element. -->
        <xsl:with-param name="value" select="@value"/>
      </xsl:call-template>
    </xsl:copy>
  </xsl:template>

  <!-- Add to type elements the range of acceptable values. -->

  <xsl:template name="type-enum">
    <xsl:param name="value"/>
    <!-- Find all values in the range and add them. -->
    <xsl:for-each select="$types/types/type-enum">
      <xsl:copy>
        <xsl:apply-templates select="@*"/>
      </xsl:copy>
    </xsl:for-each>
  </xsl:template>

  <!-- Replicate unknown elements. -->

  <xsl:template match="@*|*|node()">
    <xsl:copy>
      <xsl:apply-templates select="@*|*|node()"/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>
